import React, { useEffect, useState } from 'react'
import { ReactComponent as DeleteIcon } from '../../assets/icons/icon-delete.svg'
import { ReactComponent as Upload } from '../../assets/icons/cloud-upload.svg'

import './index.scss'

export interface DragNdropProps {
	filesSaved: File[]
	onFilesSelected: React.Dispatch<React.SetStateAction<File[]>>
	style?: React.CSSProperties
	acceptFileType?: string[]
	maxFiles: number
	disableButton?: (disable: boolean) => void
}

export const DragNdrop: React.FC<DragNdropProps> = ({
	onFilesSelected,
	style,
	acceptFileType,
	disableButton,
	filesSaved,
	maxFiles
}) => {
	const [files, setFiles] = useState<File[]>(filesSaved)

	useEffect(() => {
		if (filesSaved !== files) {
			setFiles(filesSaved)
		}
	}, [filesSaved])

	const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
		const selectedFiles = event.target.files
		if (selectedFiles && selectedFiles.length > 0) {
			const newFiles = Array.from(selectedFiles)
			const totalFiles = files.length + newFiles.length

			if (totalFiles <= maxFiles) {
				setFiles((prevFiles) => [...prevFiles, ...newFiles])
			} else {
				alert(`Você pode realizar o uploade, de no máximo ${maxFiles} arquivos.`)
			}
		}
	}

	useEffect(() => {
		setFiles([])
		if (disableButton !== undefined) disableButton(true)
	}, [])

	const handleDrop = (event: React.DragEvent<HTMLDivElement>) => {
		event.preventDefault()
		const droppedFiles = event.dataTransfer.files
		if (droppedFiles.length > 0) {
			const newFiles = Array.from(droppedFiles)
			const totalFiles = files.length + newFiles.length

			if (totalFiles <= maxFiles) {
				setFiles((prevFiles) => [...prevFiles, ...newFiles])
			} else {
				alert(`You can only upload a maximum of ${maxFiles} files.`)
			}
		}
	}

	const handleRemoveFile = (index: number) => {
		setFiles((prevFiles) => prevFiles.filter((_, i) => i !== index))
	}

	useEffect(() => {
		if (disableButton !== undefined) {
			files.length > 0 ? disableButton(false) : disableButton(true)
		}
		onFilesSelected(files)
	}, [files, onFilesSelected])

	return (
		<div className='container-drag-div'>
			<div className='drag-drop' style={{ border: '1px solid black !important' }}>
				<div
					className={`document-uploader ${files.length > 0 ? 'upload-box active' : 'upload-box'}`}
					onDrop={handleDrop}
					style={style}
					onDragOver={(event) => event.preventDefault()}
				>
					<input
						type='file'
						hidden
						id='browse'
						onChange={handleFileChange}
						accept={acceptFileType?.join(',')}
						multiple
					/>
					<label htmlFor='browse' className='uploadable'>
						<Upload></Upload>
					</label>
					<div className='upload-info' id='browse'>
						<div className='div-upload-info-text uploadable'>
							<label htmlFor='browse' className='text-info uploadable'>
								Arraste o arquivo ou clique aqui para fazer upload
							</label>
						</div>
					</div>

					{files.length > 0 && (
						<div className='file-list'>
							<div className='file-list__container'>
								{files.map((file, index) => (
									<div className='file-item' key={index}>
										<div className='file-info'>
											<p>{file.name}</p>
											{/* <p>{file.type}</p> */}
										</div>
										<div className='file-actions' onClick={() => handleRemoveFile(index)}>
											<DeleteIcon className='delete-icon' />
										</div>
									</div>
								))}
							</div>
						</div>
					)}
				</div>
			</div>
			{files.length > 0 && (
				<div className='success-file'>
					<p>{files.length} file(s) selected</p>
				</div>
			)}
		</div>
	)
}

export default DragNdrop
