/* eslint-disable @typescript-eslint/no-unused-vars */
import './index.scss'
import React, { Dispatch, SetStateAction, useEffect, useState } from 'react'
import { Button } from '../../../../../components/Button'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import DefaultModal from '../../../../../components/DefaultModal'
import { QuestionTypes } from '../../../../Common/Activities/components/card-activity'
import CheckboxInput from '../../../../../components/CheckboxInput'
import { ReactComponent as ListArrow } from '../../../../../assets/icons/list-arrow.svg'
import { CheckboxInputTypes } from '..'
import { Loader } from '../../../../../components/Loader'
import { useGetAllActivities } from '../../../../../services/core/activity/queries'
import {
	ActivitiesData,
	ActivitiesToAdd,
	ActivityDto,
	ActivityListDto,
	ActivityResponseDto
} from '../../../../../dtos/ActivityDto'
import { useCourseModuleAddActivity } from '../../../../../services/core/courses/mutations'

const defaultCheckboxInputTypes: CheckboxInputTypes[] = [
	{
		label: QuestionTypes.MULTIPLE_CHOICE,
		isChecked: false
	},
	{
		label: QuestionTypes.CHECKBOX,
		isChecked: false
	},
	{
		label: QuestionTypes.EXTERNAL_CONTENT,
		isChecked: false
	}
]

export interface AddActivityListModalProps {
	onClickCreate: (selectedType: string) => void
	moduleId: number
	courseId: number
	initialOrder: number
	activitiesList: ActivityResponseDto[]
	setAddActivitiesList: Dispatch<SetStateAction<ActivitiesData[]>>
}

const AddActivityModal: React.FC<AddActivityListModalProps> = (
	props: AddActivityListModalProps
) => {
	const [option, setOption] = useState<string>('add')
	const [isFull, setFull] = useState<boolean>(false)
	const [selectedActivities, setSelectedActivities] = useState<ActivitiesData[]>([])
	const [activitiesToAdd, setActivitiesToAdd] = useState<ActivitiesToAdd>()
	const [currentIndex, setCurrentIndex] = useState<number>(0)
	const [activities, setActivites] = useState<ActivityListDto[]>([])
	const [isDisabled, setDisabled] = useState<boolean>(true)
	const [questionTypes, setQuestionTypes] =
		useState<CheckboxInputTypes[]>(defaultCheckboxInputTypes)
	const [questionTypeSelected, setQuestionTypeSelected] = useState<string>('')

	const { setVisibility } = useModal()
	const { data, isLoading } = useGetAllActivities()
	const {
		data: moduleAddActivity,
		mutate: mutateModuleActivity,
		isPending: isPendingAddActivity
	} = useCourseModuleAddActivity(props.moduleId!, activitiesToAdd!, props.courseId)

	const onClickAdd = () => {
		setDisabled(true)
		setActivitiesToAdd(checkAndCorrectOrder(selectedActivities))

		mutateModuleActivity()
	}

	const onClickToggle = (event: React.MouseEvent) => {
		if (option !== event.currentTarget.id) {
			setDisabled(true)
			setSelectedActivities([])
			setQuestionTypeSelected('')
			setQuestionTypes(defaultCheckboxInputTypes)
		}
		setOption(event.currentTarget.id)
	}

	useEffect(() => {
		checkAndCorrectOrder(selectedActivities)
	}, [activitiesToAdd])

	useEffect(() => {
		if (moduleAddActivity !== undefined) {
			props.setAddActivitiesList(selectedActivities)
			setSelectedActivities([])
			setVisibility?.(false)
		}
	}, [moduleAddActivity])

	useEffect(() => {
		if (data) setActivites(removeDuplicate(data))
	}, [data])

	useEffect(() => {
		setSelectedActivities([])
	}, [])

	const removeDuplicate = (activtiesToCheck: ActivityDto[]): ActivityListDto[] => {
		const activityIds = new Set(props.activitiesList.map((activity) => activity.activity.id))

		const filteredActivities = activtiesToCheck.filter((activity) => !activityIds.has(activity.id))

		return filteredActivities
	}

	const checkAndCorrectOrder = (activities: ActivitiesData[]): ActivitiesToAdd => {
		const isOrderCorrect = activities.every((activity, index) => {
			const indexToUse = props.initialOrder + index
			activity.order === indexToUse + 1
		})

		if (isOrderCorrect) {
			return { activities: activities }
		}

		const correctedActivities = activities.sort((a, b) => a.order - b.order)

		correctedActivities.forEach((activity, index) => {
			activity.order = props.initialOrder + index + 1
		})

		return { activities: correctedActivities }
	}

	const onSelectAll = () => {
		if (!data) return
		isFull
			? setSelectedActivities([])
			: setSelectedActivities(
					data?.map((activity) => {
						const activities: ActivitiesData = {
							order: currentIndex!,
							activityId: activity.id
						}
						setCurrentIndex(currentIndex + 1)
						return activities
					})
				)
		isFull ? setDisabled(true) : setDisabled(false)
		setFull(!isFull)
	}

	const onCourseSelected = (value?: number) => {
		if (!value) return
		const oldActivitiesValue = [...selectedActivities]

		const index = oldActivitiesValue.findIndex((activity) => value === activity.activityId)

		if (index !== -1) {
			const newActivities = oldActivitiesValue.filter((activity) => activity.activityId !== value)
			setSelectedActivities(newActivities)
			if (newActivities.length === 0) {
				setDisabled(true)
			}
		} else {
			setSelectedActivities((prev) => [...prev, { activityId: value, order: currentIndex }])
			setDisabled(false)
			setCurrentIndex(currentIndex + 1)
		}
	}

	const onChangeCheckbox = async (event: React.ChangeEvent<HTMLInputElement>) => {
		const inputElement = event.target as HTMLInputElement
		const labelInput = inputElement.name
		const updatedTypes: CheckboxInputTypes[] = questionTypes.map((type) => {
			if (labelInput === type.label) {
				!type.isChecked === false
					? setDisabled(true)
					: (setDisabled(false), setQuestionTypeSelected(type.label))
				return { label: type.label, isChecked: !type.isChecked }
			} else return { label: type.label, isChecked: false }
		})

		setQuestionTypes(updatedTypes)
	}

	const isOptionSelected = (value?: number) => {
		if (value)
			return (
				selectedActivities.findIndex((activity) => {
					return activity.activityId === value
				}) !== -1
			)
		return false
	}

	return (
		<DefaultModal
			removegap={true}
			title='Atividades'
			content={
				<div className='content-container'>
					<div className='switch-toggle'>
						<button
							id='create'
							className={`create-toggler ${option === 'create' ? 'active' : ''}`}
							onClick={onClickToggle}
						>
							Criar
						</button>
						<button
							id='add'
							className={`add-toggler ${option === 'add' ? 'active' : ''}`}
							onClick={onClickToggle}
						>
							Adicionar
						</button>
					</div>
					<div className='select-input-div'>
						{option === 'create' ? (
							<div className='types-div'>
								{questionTypes.map((type, index) => (
									<div className='option-div' key={index}>
										<CheckboxInput
											onChange={onChangeCheckbox}
											name={type.label}
											checked={type.isChecked}
											label={type.label}
											className='checkbox-input'
										/>
									</div>
								))}
							</div>
						) : (
							<div className='existent-middle'>
								<div className='existent-all' onClick={onSelectAll}>
									<ListArrow className='existent-all-icon' />
									<p className='existent-all-text'>Selecionar todos</p>
								</div>
								<div className='existent-select'>
									{isLoading ? (
										<Loader />
									) : (
										<ul className='select-list'>
											{activities.map((course, index) => (
												<li key={index} onClick={() => onCourseSelected(course.id)}>
													<input
														className='selected-checkbox'
														type='checkbox'
														checked={isOptionSelected(course.id)}
														readOnly
													/>
													{course.name}
												</li>
											))}
										</ul>
									)}
								</div>
							</div>
						)}
					</div>
				</div>
			}
			footer={
				<div className='container-button'>
					<div className='cancel-button-div'>
						<Button variant='outlined' color='black' onClick={() => setVisibility?.(false)}>
							<div className='modal-button-text'>Cancelar</div>
						</Button>
					</div>
					<Button
						variant='filled'
						color='primary'
						disabled={isDisabled}
						onClick={() => {
							option === 'create' ? props.onClickCreate(questionTypeSelected) : onClickAdd()
						}}
					>
						<div className='modal-button-text'>
							{isPendingAddActivity ? <Loader styleLoader={'small'} /> : 'Confirmar'}
						</div>
					</Button>
				</div>
			}
		/>
	)
}

export default AddActivityModal
