import { useEffect, useState } from 'react'
import { Button } from '../../../../../components/Button'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import DefaultModal from '../../../../../components/DefaultModal'
import { UserCreateDto } from '../../../../../dtos/UserDto'
import { useUserBatchCreateMutation } from '../../../../../services/core/users/mutations'
import DragNdrop from '../../../../../components/DragNDropFile'
import './index.scss'

export const CreateUserBatchModal: React.FC = () => {
	const [usersToCreate, setUsersToCreate] = useState<UserCreateDto[]>([])
	const [file, setFile] = useState<File[]>([])
	const { setVisibility } = useModal()

	const {
		mutate: createBatchUser,
		isPending: isCreationBatchUserPending,
		isSuccess: isCreationBatchUserSuccess
	} = useUserBatchCreateMutation(usersToCreate)

	useEffect(() => {
		if (isCreationBatchUserSuccess) {
			setVisibility?.(false)
		}
	}, [isCreationBatchUserSuccess])

	useEffect(() => {
		if (usersToCreate.length > 0) createBatchUser()
	}, [usersToCreate])

	const onSaveButtonClick = async () => {
		setUsersToCreate(await readCSVFile(file[0]))
	}

	const onCancelButtonClick = () => {
		setVisibility?.(false)
	}

	const downloadTemplate = () => {
		const csvContent = `friendlyName;email;role;areaId
João Silva;joao.silva@example.com;COLLABORATOR;1
Maria Souza;maria.souza@example.com;ADMIN;3
Pedro Oliveira;pedro.oliveira@example.com;COLLABORATOR;2`
		const url = window.URL.createObjectURL(new Blob([csvContent], { type: 'text/csv' }))

		const link = document.createElement('a')
		link.href = url
		link.setAttribute('download', 'user-template.csv')
		document.body.appendChild(link)
		link.click()
		link.parentNode?.removeChild(link)
	}

	const saveButtonDisabled = file.length === 0 || isCreationBatchUserPending

	const readCSVFile = (file: File): Promise<UserCreateDto[]> => {
		return new Promise((resolve, reject) => {
			const reader = new FileReader()

			reader.onload = (event) => {
				const text = event.target?.result as string
				const lines = text.split('\n')

				// Verifica se há linhas no arquivo
				if (lines.length === 0) {
					reject(new Error('O arquivo CSV está vazio.'))
					return
				}

				// Extrai os cabeçalhos do CSV (assumindo que estão na primeira linha)
				const headers = lines[0].split(';').map((header) => header.trim())

				// Verifica se os cabeçalhos obrigatórios estão presentes
				const requiredHeaders = ['email', 'friendlyName', 'role']
				const hasRequiredHeaders = requiredHeaders.every((header) => headers.includes(header))

				if (!hasRequiredHeaders) {
					reject(new Error('O arquivo CSV deve conter os cabeçalhos: email, friendlyName, role'))
					return
				}

				// Processa cada linha de dados (começa a partir da segunda linha)
				const results: UserCreateDto[] = []
				for (let i = 1; i < lines.length; i++) {
					const line = lines[i].trim()
					if (line === '') continue

					const values = line.split(';').map((value) => value.trim())
					if (values.length !== headers.length) {
						reject(new Error(`Erro de formatação na linha ${i + 1} do CSV.`))
						return
					}

					const record: Partial<UserCreateDto> = {}
					headers.forEach((header, index) => {
						if (requiredHeaders.includes(header) || header === 'areaId') {
							if (header in record) {
								;(record as Record<string, string>)[header] = values[index]
							}
						}
					})

					results.push(record as UserCreateDto)
				}

				resolve(results)
			}

			reader.onerror = (error) => {
				reject(error)
			}

			reader.readAsText(file)
		})
	}

	return (
		<DefaultModal
			title='Importar contas por arquivo'
			subtitle='Certifique-se de fazer igual o template'
			content={
				<>
					<div className='user-batch-content'>
						<div className='template-container'>
							<Button color='black' variant='outlined' onClick={downloadTemplate}>
								<p className='button-template'>Template CSV</p>
							</Button>
						</div>

						<DragNdrop
							maxFiles={1}
							filesSaved={file}
							acceptFileType={['.csv']}
							onFilesSelected={setFile}
							style={{ width: '100%', height: '15rem' }}
						></DragNdrop>
					</div>
				</>
			}
			footer={
				<div className='footer-modal'>
					<Button onClick={onCancelButtonClick} color='secondary'>
						<p className='cancel'>Cancelar</p>
					</Button>
					<Button onClick={onSaveButtonClick} color='primary'>
						<p className='confirm'>Confirmar</p>
					</Button>
				</div>
			}
		></DefaultModal>
	)
}

export default CreateUserBatchModal
