import { Button } from '../../../../../components/Button'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import Input from '../../../../../components/Input'
import './index.scss'
import React, { useState, useEffect } from 'react'
import SelectInput from '../../../../../components/SelectInput'
import DefaultModal from '../../../../../components/DefaultModal'
import { UserRole } from '../../../../../dtos/UserDto'
import { useUserCreateMutation } from '../../../../../services/core/users/mutations'
import { useGetAllAreas } from '../../../../../services/core/areas/queries'
import { AreaDto } from '../../../../../dtos/AreaDto'

export interface CreateUserModalProps {
	role?: UserRole
}

export const CreateUserModal: React.FC<CreateUserModalProps> = ({ role }) => {
	const [email, setEmail] = useState('')
	const [name, setName] = useState('')
	const { setVisibility } = useModal()
	const [selectedAreaID, setSelectedAreaID] = useState<number>()
	const [areas, setAreas] = useState<AreaDto[]>([])
	const { data: areasData } = useGetAllAreas()

	const {
		mutate: createUser,
		isPending: isCreationPending,
		isSuccess: isCreationSuccess
	} = useUserCreateMutation({
		email,
		friendlyName: name,
		role: role?.toString() || '',
		areaId: selectedAreaID || 0
	})

	useEffect(() => {
		if (areasData) setAreas(areasData)
	}, [areasData])

	useEffect(() => {
		if (isCreationSuccess) {
			resetForm()
			setVisibility?.(false)
		}
	}, [isCreationSuccess])

	const resetForm = () => {
		setName('')
		setEmail('')
		setSelectedAreaID(0)
	}

	const saveButtonDisabled = !name || !email || !selectedAreaID || isCreationPending

	const onSaveButtonClick = () => {
		createUser()
	}

	const onCancelButtonClick = () => {
		setVisibility?.(false)
	}

	const handleEmailChange = (event: React.ChangeEvent<HTMLInputElement>) => {
		setEmail(event.target.value)
	}

	const handleNameChange = (event: React.ChangeEvent<HTMLInputElement>) => {
		setName(event.target.value)
	}

	const onSelectArea = (value?: number) => {
		if (value) setSelectedAreaID(value)
	}

	const isAreaSelected = (value?: number) => {
		return value === selectedAreaID
	}

	return (
		<DefaultModal
			title={role === UserRole.ADMIN ? 'Criar conta de RH' : 'Criar conta de colaborador'}
			subtitle='Preencha os dados do novo integrante de sua equipe.'
			content={
				<>
					<div className='user-modal-content'>
						<div className='inputs-modal'>
							<div className='input-name'>
								<Input label='Nome' value={name} onChange={handleNameChange} />
							</div>
							<div className='input-email'>
								<Input label='Email' value={email} onChange={handleEmailChange} />
							</div>
							<div className='select-area'>
								<SelectInput
									label='Selecione a área:'
									options={areas.map((area) => ({ label: area.name, value: area.id }))}
									onSelectOption={onSelectArea}
									isOptionSelected={isAreaSelected}
									value={areas.find((area) => area.id === selectedAreaID)?.name}
								/>
							</div>
						</div>
					</div>
				</>
			}
			footer={
				<>
					<Button variant='filled' color='secondary' onClick={onCancelButtonClick}>
						<p className='modal-button-text'>Cancelar</p>
					</Button>

					<Button
						disabled={saveButtonDisabled}
						variant='filled'
						color='primary'
						onClick={onSaveButtonClick}
					>
						<p className='modal-button-text'>Salvar</p>
					</Button>
				</>
			}
		/>
	)
}

export default CreateUserModal
