import { Button } from '../../../../../components/Button'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import Input from '../../../../../components/Input'
import './index.scss'
import React, { useState, useEffect } from 'react'
import SelectInput from '../../../../../components/SelectInput'
import DefaultModal from '../../../../../components/DefaultModal'
import { UserDto, UserRole } from '../../../../../dtos/UserDto'
import { useUserUpdateMutation } from '../../../../../services/core/users/mutations'
import { useGetAllAreas } from '../../../../../services/core/areas/queries'
import { AreaDto } from '../../../../../dtos/AreaDto'

export interface UpdateModalProps {
	user: UserDto
}

export const UpdateUserModal: React.FC<UpdateModalProps> = ({ user }) => {
	const [email, setEmail] = useState(user.email)
	const [friendlyName, setFriendlyName] = useState(user.friendlyName)
	const { setVisibility, hidden } = useModal()
	const [selectedAreaID, setSelectedAreaID] = useState<number>(user.area.id)
	const [areas, setAreas] = useState<AreaDto[]>([])

	useEffect(() => {
		setEmail(user.email)
		setFriendlyName(user.friendlyName)
		setSelectedAreaID(user.area.id)
	}, [hidden, user])

	const { data: areasData } = useGetAllAreas()

	const {
		mutate: updateMutate,
		isPending: isUpdatePending,
		isSuccess: isUpdateSuccess
	} = useUserUpdateMutation({
		email,
		friendlyName,
		area: areas.find((area) => area.id === selectedAreaID) || user.area,
		role: user?.role || UserRole.COLLABORATOR,
		id: user.id
	})

	useEffect(() => {
		if (areasData) setAreas(areasData)
	}, [areasData])

	useEffect(() => {
		if (isUpdateSuccess) {
			setVisibility?.(false)
		}
	}, [isUpdateSuccess])

	const onSaveButtonClick = () => {
		updateMutate()
		setVisibility?.(false)
	}

	const onCancelButtonClick = () => {
		setVisibility?.(false)
	}

	const handleEmailChange = (event: React.ChangeEvent<HTMLInputElement>) => {
		setEmail(event.target.value)
	}

	const handleNameChange = (event: React.ChangeEvent<HTMLInputElement>) => {
		setFriendlyName(event.target.value)
	}

	const onSelectArea = (value?: number) => {
		if (value) setSelectedAreaID(value)
	}

	const isAreaSelected = (value?: number) => {
		return value === selectedAreaID
	}

	return (
		<DefaultModal
			title={
				user.role === UserRole.ADMIN ? 'Atualizar conta de RH' : 'Atualizar conta de colaborador'
			}
			subtitle='Preencha com os novos dados integrante de sua equipe.'
			content={
				<>
					<div className='user-modal-content'>
						<div className='inputs-modal'>
							<div className='input-name'>
								<Input label='Nome' value={friendlyName} onChange={handleNameChange} />
							</div>
							<div className='input-email'>
								<Input label='Email' value={email} onChange={handleEmailChange} />
							</div>
							<div className='select-area'>
								<SelectInput
									label='Selecione a área:'
									options={areas.map((area) => ({ label: area.name, value: area.id }))}
									onSelectOption={onSelectArea}
									isOptionSelected={isAreaSelected}
									value={areas.find((area) => area.id === selectedAreaID)?.name}
								/>
							</div>
						</div>
					</div>
				</>
			}
			footer={
				<>
					<Button variant='filled' color='secondary' onClick={onCancelButtonClick}>
						<p className='modal-button-text'>Cancelar</p>
					</Button>

					<Button
						disabled={
							(user.friendlyName === friendlyName && user.email === email) || isUpdatePending
						}
						variant='filled'
						color='primary'
						onClick={onSaveButtonClick}
					>
						<p className='modal-button-text'>Salvar</p>
					</Button>
				</>
			}
		/>
	)
}

export default UpdateUserModal
