import React, { useEffect, useState } from 'react'
import './index.scss'
import Table from '../../../components/Table'
import { Button } from '../../../components/Button'
import { useModal } from '../../../providers/Modal/ModalProvider'
import { CreateUserModal } from './components/CreateUserModal'
import { UpdateUserModal } from './components/UpdateUserModal'
import { useGetAllUsers } from '../../../services/core/users/queries'
import { UserDto, UserRole } from '../../../dtos/UserDto'
import SelectInput from '../../../components/SelectInput'
import Input from '../../../components/Input'
import Confirmation from '../../../components/Confirmation'
import { useUserDeleteMutation } from '../../../services/core/users/mutations'
import { useDownloadReport } from '../../../services/core/report/queries'
import CreateUserBatchModal from './components/CreateUserBatchModal'
import { AreaDto } from '../../../dtos/AreaDto'
import { useGetAllAreas } from '../../../services/core/areas/queries'

const Admin: React.FC = () => {
	const { setModalContent, setVisibility } = useModal()
	const [selectedHRAreaID, setSelectedHRAreaID] = useState<number>()
	const [selectedColabAreaID, setSelectedColabAreaID] = useState<number>()
	const [HRFilterName, setHRFilterName] = useState<string>('')
	const [colabFilterName, setColabFilterName] = useState<string>('')
	const [adminUsers, setAdminUsers] = useState<UserDto[]>([])
	const [collaboratorUsers, setCollaboratorUsers] = useState<UserDto[]>([])
	const [currentIndexToDelete, setCurrentIndexToDelete] = useState<number>(0)
	const [currentRoleToDelete, setCurrentRoleToDelete] = useState<UserRole>(UserRole.COLLABORATOR)
	const [selectedUser, setSelectedUser] = useState<{ id: number } | undefined>(undefined)
	const [areas, setAreas] = useState<AreaDto[]>([])
	const { data: areasData } = useGetAllAreas()
	const { data: adminUsersData } = useGetAllUsers(UserRole.ADMIN)
	const { data: collaboratorUsersData } = useGetAllUsers(UserRole.COLLABORATOR)
	useEffect(() => {
		if (adminUsersData) setAdminUsers(adminUsersData)
		if (collaboratorUsersData) setCollaboratorUsers(collaboratorUsersData)
	}, [adminUsersData, collaboratorUsersData])

	useEffect(() => {
		if (areasData) setAreas(areasData)
	}, [areasData])

	const { mutate: deleteMutation, isSuccess: isDeleteSuccess } = useUserDeleteMutation(
		selectedUser?.id
	)

	const onSelectHRArea = (value?: number) => {
		if (value) setSelectedHRAreaID(value)
	}

	const isHRAreaSelected = (value?: number) => {
		return value === selectedHRAreaID
	}

	const onSelectColabArea = (value?: number) => {
		if (value) setSelectedColabAreaID(value)
	}

	const isColabAreaSelected = (value?: number) => {
		return value === selectedColabAreaID
	}

	const optionsArea = areas.map((area) => {
		return { label: area.name, value: area.id }
	})

	const openCreateAdminModal = () => {
		if (setModalContent && setVisibility) {
			setModalContent(<CreateUserModal role={UserRole.ADMIN} />)
			setVisibility(true)
		}
	}

	const openCreateCollaboratorModal = () => {
		if (setModalContent && setVisibility) {
			setModalContent(<CreateUserModal role={UserRole.COLLABORATOR} />)
			setVisibility(true)
		}
	}

	const openCreateBatchUserModal = () => {
		if (setModalContent && setVisibility) {
			setModalContent(<CreateUserBatchModal />)
			setVisibility(true)
		}
	}

	const openUpdateModal = (user: UserDto) => {
		if (setModalContent && setVisibility) {
			setModalContent(<UpdateUserModal user={user} />)
			setVisibility(true)
		}
	}

	useEffect(() => {
		if (currentRoleToDelete === UserRole.ADMIN) {
			if (adminUsersData !== undefined) {
				adminUsersData?.splice(currentIndexToDelete, 1)
				setAdminUsers(adminUsersData)
			}
		}
		if (currentRoleToDelete === UserRole.COLLABORATOR) {
			if (collaboratorUsersData !== undefined) {
				collaboratorUsersData?.splice(currentIndexToDelete, 1)
				setCollaboratorUsers(collaboratorUsersData)
			}
		}
	}, [isDeleteSuccess])

	const openDeleteModal = (user: UserDto, index: number, role: UserRole) => {
		setCurrentRoleToDelete(role)
		setCurrentIndexToDelete(index)
		setSelectedUser(user)
		if (setModalContent && setVisibility) {
			setModalContent(
				<Confirmation
					title='Excluir usuário'
					description={`Deseja excluir o usuário ${user.friendlyName}?`}
					handleConfirm={() => {
						deleteMutation()
					}}
				/>
			)
			setVisibility(true)
		}
	}

	return (
		<div className='admin-page-container'>
			<div className='admin-header-text'>
				<h1 className='admin-title'>Administração</h1>
				<h3 className='admin-subtitle'>
					Aqui fica o gerenciamento de colaboradores: Acompanhe o progresso e expanda sua equipe com
					facilidade
				</h3>
			</div>
			<div className='container-relatorio'>
				<Button color='black' variant='outlined' onClick={useDownloadReport}>
					<p className='button-relatorio'>Baixar Relatório</p>
				</Button>
				<Button color='black' variant='outlined' onClick={openCreateBatchUserModal}>
					<p className='button-import'>Importar usuários</p>
				</Button>
			</div>
			<div className='container-co-table'>
				<h2 className='co-table-title'>COLABORADORES</h2>
				<p className='co-table-label'>sobre os colaboradores</p>
				<div className='container-admin-filters'>
					<SelectInput
						label='Selecione a área:'
						options={optionsArea}
						onSelectOption={onSelectColabArea}
						isOptionSelected={isColabAreaSelected}
						value={optionsArea.find((o) => o.value === selectedColabAreaID)?.label}
					/>
					<Input
						label='Pesquisar por nome'
						type='text'
						value={colabFilterName}
						onChange={(e) => setColabFilterName(e.target.value)}
					/>
				</div>
				<div className='container-collaborator-table'>
					{collaboratorUsers.length > 0 ? (
						<div className='table-collaborator'>
							<Table
								headerItems={['Nome', 'Email', 'Area']}
								bodyItems={[
									...collaboratorUsers
										.filter((user) => {
											return colabFilterName.toLowerCase() === ''
												? user
												: user.friendlyName.toLowerCase().includes(colabFilterName.toLowerCase())
										})
										.map((user, index) => [
											user.friendlyName,
											user.email,
											user.area ? user.area.name : 'Nulo',
											<Button
												key={index}
												variant='text'
												color='primary'
												onClick={() => openUpdateModal(user)}
											>
												<p className='button-add-text'>Editar</p>
											</Button>,
											<Button
												key={index}
												variant='text'
												color='primary'
												onClick={() => openDeleteModal(user, index, UserRole.COLLABORATOR)}
											>
												<p className='button-add-text'>Remover</p>
											</Button>
										])
								]}
							/>
						</div>
					) : (
						<p>Nenhum colaborador encontrado</p>
					)}
					<div className='table-button-container'>
						<Button variant='text' color='primary' onClick={openCreateCollaboratorModal}>
							<p className='button-add-text'>Adicionar +</p>
						</Button>
					</div>
				</div>
			</div>
			<hr className='table-divisor' />
			<div className='container-rh-full-table'>
				<h2 className='rh-table-title'>RH</h2>
				<p className='rh-table-label'>sobre os RH</p>
				<div className='container-admin-HR-filters'>
					<SelectInput
						label='Selecione a área:'
						options={optionsArea}
						onSelectOption={onSelectHRArea}
						isOptionSelected={isHRAreaSelected}
						value={optionsArea.find((o) => o.value === selectedHRAreaID)?.label}
					/>
					<Input
						label='Pesquisar por nome'
						type='text'
						value={HRFilterName}
						onChange={(e) => setHRFilterName(e.target.value)}
					/>
				</div>
				<div className='container-table-data-rh'>
					{adminUsers.length > 0 ? (
						<div className='table-data-rh'>
							<Table
								headerItems={['Nome', 'Email', 'Area']}
								bodyItems={[
									...adminUsers
										.filter((user) => {
											return HRFilterName.toLowerCase() === ''
												? user
												: user.friendlyName.toLowerCase().includes(HRFilterName.toLowerCase())
										})
										.map((user, index) => [
											user.friendlyName,
											user.email,
											user.area ? user.area.name : 'Nulo',
											<Button
												key={index}
												variant='text'
												color='primary'
												onClick={() => openUpdateModal(user)}
											>
												<p className='button-add-text'>Editar</p>
											</Button>,
											<Button
												key={index}
												variant='text'
												color='primary'
												onClick={() => openDeleteModal(user, index, UserRole.ADMIN)}
											>
												<p className='button-add-text'>Remover</p>
											</Button>
										])
								]}
							/>
						</div>
					) : (
						<p>Nenhum usuário de RH encontrado</p>
					)}
					<div className='table-button-container'>
						<Button variant='text' color='primary' onClick={openCreateAdminModal}>
							<p className='button-add-text'>Adicionar +</p>
						</Button>
					</div>
				</div>
			</div>
		</div>
	)
}

export default Admin
