import React, { useEffect, useState } from 'react'
import { Button } from '../../../../../components/Button'
import Input from '../../../../../components/Input'
import ImageInput from '../../../../../components/ImageInput'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import './index.scss'
import { AreaDto } from '../../../../../dtos/AreaDto'
import {
	useAreaCreateMutation,
	useAreaUpdateMutation,
	useAreaDeleteMutation
} from '../../../../../services/core/areas/mutations'
import { Loader } from '../../../../../components/Loader'
import DefaultModal from '../../../../../components/DefaultModal'
import { placeholder } from '../../../../../assets/placeholder/placeholder'
import Confirmation from '../../../../../components/Confirmation'
interface ModalAreaProps {
	modalTitle: string
	area?: AreaDto
	isEdit?: boolean
}

const AreaModal: React.FC<ModalAreaProps> = ({ area, isEdit, modalTitle }) => {
	const { setModalContent, setVisibility, hidden } = useModal()
	const [areaName, setAreaName] = useState(area?.name || '')
	const [areaImage, setAreaImage] = useState<File | string>(
		area?.imageUrl ? new File([area.imageUrl], 'areaImage.png') : placeholder
	)

	const creationMutate = useAreaCreateMutation()
	const updateMutate = useAreaUpdateMutation()
	const { mutate: deleteMutate, isPending: isDeletePending } = useAreaDeleteMutation(area?.id)

	useEffect(() => {
		if (area) {
			setAreaName(area.name)
			setAreaImage(area.imageUrl)
			return
		}
		resetForm()
	}, [hidden])

	useEffect(() => {
		if (creationMutate.isSuccess || updateMutate.isSuccess) {
			resetForm()
			setVisibility?.(false)
		}
	}, [creationMutate.isSuccess, updateMutate.isSuccess])

	const resetForm = () => {
		setAreaName('')
		setAreaImage(placeholder)
	}

	const base64ToFile = (base64: string, filename: string): File => {
		const arr = base64.split(',')
		const mime = arr[0].match(/:(.*?);/)?.[1] || ''
		const bstr = atob(arr[1])
		let n = bstr.length
		const u8arr = new Uint8Array(n)
		while (n--) {
			u8arr[n] = bstr.charCodeAt(n)
		}
		return new File([u8arr], filename, { type: mime })
	}

	const handleSubmit = () => {
		const payload = new FormData()
		payload.append('name', areaName)

		if (typeof areaImage === 'string' && areaImage.startsWith('data:image')) {
			const file = base64ToFile(areaImage, 'areaImage.png')
			payload.append('file', file)
		} else if (areaImage instanceof File) {
			payload.append('file', areaImage)
		}

		if (isEdit) {
			payload.append('id', String(area?.id || 0))
			updateMutate.mutate(payload)
		} else {
			creationMutate.mutate(payload)
		}
	}

	const handleImageChange = (file: File) => {
		const fileReader = new FileReader()
		fileReader.onloadend = () => {
			setAreaImage(fileReader.result as string)
		}
		fileReader.readAsDataURL(file)
	}

	const isConfirmButtonDisabled =
		!areaName || creationMutate.isPending || updateMutate.isPending || isDeletePending

	const handleConfirmation = () => {
		setModalContent?.(
			<Confirmation
				title='Deletar Curso'
				description='Tem certeza que quer deletar este curso?'
				handleConfirm={deleteMutate}
			/>
		)
		setVisibility?.(true)
	}

	return (
		<DefaultModal
			title={modalTitle}
			subtitle='Personalize o acesso de cada colaborador.'
			content={
				<div className='area-modal-content'>
					<ImageInput handleImageChange={handleImageChange} imageUrl={areaImage} />
					<div className='form-column'>
						<Input
							type='text'
							label='Título'
							value={areaName}
							onChange={(e) => setAreaName(e.target.value)}
						/>
					</div>
				</div>
			}
			footer={
				<>
					{isEdit ? (
						<Button variant='outlined' color='transparent' onClick={handleConfirmation}>
							<div className='area-button-text'>Deletar</div>
						</Button>
					) : (
						<Button variant='outlined' color='transparent' onClick={() => setVisibility?.(false)}>
							<div className='modal-button-text'>Cancelar</div>
						</Button>
					)}
					<Button
						disabled={isConfirmButtonDisabled}
						variant='filled'
						color='primary'
						onClick={handleSubmit}
					>
						{creationMutate.isPending || updateMutate.isPending ? (
							<div className='modal-button-loader'>
								<Loader />
							</div>
						) : (
							<div className='modal-button-text'>Salvar</div>
						)}
					</Button>
				</>
			}
		/>
	)
}
export default AreaModal
