import React, { useEffect, useState } from 'react'
import { Button } from '../../../../../components/Button'
import Input from '../../../../../components/Input'
import ImageInput from '../../../../../components/ImageInput'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import './index.scss'
import { CourseDto } from '../../../../../dtos/CourseDto'
import { Loader } from '../../../../../components/Loader'
import DefaultModal from '../../../../../components/DefaultModal'
import Confirmation from '../../../../../components/Confirmation'
import {
	useCourseCreateMutation,
	useCourseDeleteMutation,
	useCourseUpdateMutation
} from '../../../../../services/core/courses/mutations'
import { placeholder } from '../../../../../assets/placeholder/placeholder'

interface ModalCourseProps {
	modalTitle: string
	course?: CourseDto
	isEdit?: boolean
}

const CourseModal: React.FC<ModalCourseProps> = ({ course, isEdit, modalTitle }) => {
	const { setModalContent, setVisibility, hidden } = useModal()
	const [courseName, setCourseName] = useState(course?.name || '')
	const [courseDescription, setCourseDescription] = useState(course?.description || '')
	const [courseImage, setCourseImage] = useState<File | string>(
		course?.imageUrl ? new File([course.imageUrl], 'courseImage.png') : placeholder
	)

	const creationMutate = useCourseCreateMutation()
	const updateMutate = useCourseUpdateMutation()
	const { mutate: deleteMutate, isPending: isDeletePending } = useCourseDeleteMutation(course?.id)

	useEffect(() => {
		if (course) {
			setCourseName(course.name)
			setCourseDescription(course.description)
			setCourseImage(course.imageUrl)
			return
		}
		resetForm()
	}, [hidden])

	useEffect(() => {
		if (creationMutate.isSuccess || updateMutate.isSuccess) {
			resetForm()
			setVisibility?.(false)
		}
	}, [creationMutate.isSuccess, updateMutate.isSuccess])

	const base64ToFile = (base64: string, filename: string): File => {
		const arr = base64.split(',')
		const mime = arr[0].match(/:(.*?);/)?.[1] || ''
		const bstr = atob(arr[1])
		let n = bstr.length
		const u8arr = new Uint8Array(n)
		while (n--) {
			u8arr[n] = bstr.charCodeAt(n)
		}
		return new File([u8arr], filename, { type: mime })
	}

	const handleSubmit = () => {
		const payload = new FormData()
		payload.append('description', courseDescription)
		payload.append('name', courseName)
		payload.append('courseTypeId', '1000')

		if (typeof courseImage === 'string' && courseImage.startsWith('data:image')) {
			const file = base64ToFile(courseImage, 'courseImage.png')
			payload.append('file', file)
		} else if (courseImage instanceof File) {
			payload.append('file', courseImage)
		}

		if (isEdit) {
			payload.append('id', String(course?.id || 0))
			updateMutate.mutate(payload)
		} else {
			creationMutate.mutate(payload)
		}
	}

	const resetForm = () => {
		setCourseName('')
		setCourseDescription('')
		setCourseImage(placeholder)
	}

	const handleImageChange = (file: File) => {
		const fileReader = new FileReader()
		fileReader.onloadend = () => {
			setCourseImage(fileReader.result as string)
		}
		fileReader.readAsDataURL(file)
	}

	const isConfirmButtonDisabled =
		!courseName ||
		!courseDescription ||
		creationMutate.isPending ||
		updateMutate.isPending ||
		isDeletePending

	const handleConfirmation = () => {
		setModalContent?.(
			<Confirmation
				title='Deletar Curso'
				description='Tem certeza que quer deletar este curso?'
				handleConfirm={deleteMutate}
			/>
		)
		setVisibility?.(true)
	}

	return (
		<DefaultModal
			title={modalTitle}
			subtitle='Personalize o curso disponibilizado aos colaboradores.'
			content={
				<>
					<ImageInput handleImageChange={handleImageChange} imageUrl={courseImage} />
					<div className='form-column'>
						<Input
							type='text'
							label='Título'
							value={courseName}
							onChange={(e) => setCourseName(e.target.value)}
						/>
						<Input
							type='text'
							label='Descrição'
							value={courseDescription}
							onChange={(e) => setCourseDescription(e.target.value)}
						/>
					</div>
				</>
			}
			footer={
				<>
					{isEdit ? (
						<Button variant='outlined' color='transparent' onClick={handleConfirmation}>
							<div className='modal-button-text'>Deletar</div>
						</Button>
					) : (
						<Button variant='outlined' color='transparent' onClick={() => setVisibility?.(false)}>
							<div className='modal-button-text'>Cancelar</div>
						</Button>
					)}
					<Button
						disabled={isConfirmButtonDisabled}
						variant='filled'
						color='primary'
						onClick={handleSubmit}
					>
						{creationMutate.isPending || updateMutate.isPending ? (
							<div className='modal-button-loader'>
								<Loader />
							</div>
						) : (
							<div className='modal-button-text'>Salvar</div>
						)}
					</Button>
				</>
			}
		/>
	)
}

export default CourseModal
