import './index.scss'
import { useEffect, useState } from 'react'
import { Card } from '../../../components/Card'
import CourseModal from './components/CourseModal'
import CourseModalExistent from './components/CourseModalExistent'
import { useModal } from '../../../providers/Modal/ModalProvider'
import { Button, ButtonProps } from '../../../components/Button'
import { ReactComponent as EditPencil } from '../../../assets/icons/pecil.svg'
import { CourseDto } from '../../../dtos/CourseDto'
import { Loader } from '../../../components/Loader'
import { useSearchParams } from 'react-router-dom'
import { DragEndEvent } from '@dnd-kit/core'
import { arrayMove } from '@dnd-kit/sortable'
import DraggableSlider from '../../../components/DraggableSlider'
import { useAreaCourseChangeOrderMutation } from '../../../services/core/areas/mutations'
import { useNavigation } from '../../../providers/Navigation/NavigationProvider'
import { useGetAllCoursesWithArea } from '../../../services/core/courses/queries'

const OrganizationCourse: React.FC = () => {
	const { navigateModules } = useNavigation()
	const { setModalContent, setVisibility } = useModal()
	const [searchParams] = useSearchParams()
	const [areaId, setAreaId] = useState<number>()
	const [courses, setCourses] = useState<CourseDto[]>([])
	const [isEditingOrder, setIsEditingOrder] = useState<boolean>(false)
	const [originalOrder, setOriginalOrder] = useState<CourseDto[]>([])

	const { data, isLoading } = useGetAllCoursesWithArea(areaId)

	const {
		mutate: changeOrderMutate,
		isPending: isChangeOrderPending,
		isSuccess: isChangeOrderSuccess
	} = useAreaCourseChangeOrderMutation(areaId, courses)

	useEffect(() => {
		if (!isEditingOrder) setOriginalOrder([...courses])
	}, [courses, isEditingOrder])

	useEffect(() => {
		if (searchParams.has('areaId')) setAreaId(parseInt(searchParams.get('areaId') as string))
	}, [searchParams])

	useEffect(() => {
		if (data) setCourses(data)
	}, [data])

	useEffect(() => {
		if (isChangeOrderSuccess) setIsEditingOrder(false)
	}, [isChangeOrderSuccess])

	const handleCreate = () => {
		setModalContent?.(<CourseModal modalTitle='Criar Curso' />)
		setVisibility?.(true)
	}

	const handleEdit = (course: CourseDto) => {
		setModalContent?.(<CourseModal modalTitle='Editando Curso' course={course} isEdit={true} />)
		setVisibility?.(true)
	}

	const handleOrderCancel = () => {
		setCourses([...originalOrder])
		setIsEditingOrder(false)
	}

	const handleAddExistent = () => {
		setModalContent?.(<CourseModalExistent areaId={Number(areaId)} />)
		setVisibility?.(true)
	}

	const getCoursePos = (id: number) => courses.findIndex((m) => m.id === id)

	const handleDragEnd = (event: DragEndEvent) => {
		setIsEditingOrder(true)
		const { active, over } = event

		if (!over) return
		if (active.id === over.id) return

		const originalPos = getCoursePos(parseInt(active.id as string))
		const newPos = getCoursePos(parseInt(over.id as string))

		const newArray = arrayMove(courses, originalPos, newPos)

		setCourses(newArray)
	}

	const areButtonsDisabled = isEditingOrder

	const renderCard = (item: CourseDto) => {
		const buttons: ButtonProps[] = [
			{
				variant: 'filled',
				color: 'primary',
				disabled: areButtonsDisabled,
				onClick: () => navigateModules?.({ courseId: item.id }),
				children: (
					<div className='card-button-div'>
						<p className='card-button-text'>Entrar</p>
					</div>
				)
			},
			{
				variant: 'filled',
				color: 'gray',
				disabled: areButtonsDisabled,
				onClick: () => handleEdit(item),
				children: (
					<div className='card-button-div'>
						<EditPencil className='card-button-icon' />
						<p className='card-button-text'>Editar</p>
					</div>
				)
			}
		]
		return <Card title={item.name} imageUrl={item.imageUrl} buttons={buttons} />
	}

	return (
		<div className='course-container'>
			<div className='course-top'>
				<h1 className='course-title'>Cursos</h1>
				<p className='course-text'>
					Personalize os Títulos e Descrições dos cursos oferecidos aos Colaboradores.
				</p>
			</div>
			<div className='course-middle'>
				<div className='course-cards'>
					{isLoading ? (
						<Loader />
					) : (
						<DraggableSlider
							items={courses}
							handleDragEnd={handleDragEnd}
							renderItem={renderCard}
						/>
					)}
				</div>
				<div className='course-bottom'>
					<div className='course-button'>
						{isEditingOrder ? (
							<>
								<Button
									variant='outlined'
									color='transparent'
									size='big'
									onClick={handleOrderCancel}
									disabled={isChangeOrderPending}
								>
									<p className='course-button-text-add'>Cancelar</p>
								</Button>
								<Button
									disabled={isChangeOrderPending}
									variant='filled'
									color='primary'
									size='big'
									onClick={changeOrderMutate}
								>
									{isChangeOrderPending ? (
										<Loader />
									) : (
										<p className='course-button-text-add'>Salvar ordem</p>
									)}
								</Button>
							</>
						) : (
							<>
								<Button
									disabled={areButtonsDisabled}
									variant='filled'
									color='primary'
									size='big'
									onClick={handleAddExistent}
								>
									<p className='course-button-text-add'>Adicionar Existente</p>
								</Button>

								<Button
									disabled={areButtonsDisabled}
									variant='filled'
									color='primary'
									size='big'
									onClick={handleCreate}
								>
									<p className='course-button-text-add'>Criar</p>
								</Button>
							</>
						)}
					</div>
				</div>
			</div>
		</div>
	)
}

export default OrganizationCourse
