import React, { useEffect, useState } from 'react'
import { Button } from '../../../../../components/Button'
import Input from '../../../../../components/Input'
import ImageInput from '../../../../../components/ImageInput'
import { useModal } from '../../../../../providers/Modal/ModalProvider'
import './index.scss'
import { ModuleDto } from '../../../../../dtos/ModuleDto'

import { Loader } from '../../../../../components/Loader'
import DefaultModal from '../../../../../components/DefaultModal'
import Confirmation from '../../../../../components/Confirmation'
import {
	useCourseModuleCreateMutation,
	useCourseModuleDeleteMutation,
	useCourseModuleUpdateMutation
} from '../../../../../services/core/courses/mutations'
import { placeholder } from '../../../../../assets/placeholder/placeholder'
interface ModalModuleProps {
	modalTitle: string
	module?: ModuleDto
	courseId?: number
	isEdit?: boolean
}

const ModuleModal: React.FC<ModalModuleProps> = ({ module, isEdit, modalTitle, courseId }) => {
	const { setModalContent, setVisibility, hidden } = useModal()
	const [moduleName, setModuleName] = useState(module?.name || '')
	const [moduleDescription, setModuleDescription] = useState(module?.description || '')
	const [moduleImage, setModuleImage] = useState<File | string>(
		module?.imageUrl ? new File([module.imageUrl], 'moduleImage.png') : placeholder
	)

	const creationMutate = useCourseModuleCreateMutation()
	const updateMutate = useCourseModuleUpdateMutation()

	const { mutate: deleteMutate, isPending: isDeletePending } = useCourseModuleDeleteMutation(
		module?.courseId,
		module?.id
	)

	useEffect(() => {
		if (module) {
			setModuleName(module.name)
			setModuleDescription(module.description)
			setModuleImage(module.imageUrl)
			return
		}
		resetForm()
	}, [hidden])

	useEffect(() => {
		if (creationMutate.isSuccess || updateMutate.isSuccess) {
			resetForm()
			setVisibility?.(false)
		}
	}, [creationMutate.isSuccess || updateMutate.isSuccess])

	const resetForm = () => {
		setModuleName('')
		setModuleDescription('')
		setModuleImage(placeholder)
	}

	const handleImageChange = (file: File) => {
		const fileReader = new FileReader()
		fileReader.onloadend = () => {
			setModuleImage(fileReader.result as string)
		}
		fileReader.readAsDataURL(file)
	}

	const handleConfirmation = () => {
		setModalContent?.(
			<Confirmation
				title='Deletar Módulo'
				description='Tem certeza que quer deletar este módulo?'
				handleConfirm={deleteMutate}
			/>
		)
		setVisibility?.(true)
	}

	const base64ToFile = (base64: string, filename: string): File => {
		const arr = base64.split(',')
		const mime = arr[0].match(/:(.*?);/)?.[1] || ''
		const bstr = atob(arr[1])
		let n = bstr.length
		const u8arr = new Uint8Array(n)
		while (n--) {
			u8arr[n] = bstr.charCodeAt(n)
		}
		return new File([u8arr], filename, { type: mime })
	}

	const handleSubmit = () => {
		const payload = new FormData()
		payload.append('description', moduleDescription)
		payload.append('name', moduleName)
		payload.append('courseTypeId', '1000')
		payload.append('courseId', String(courseId))

		if (typeof moduleImage === 'string' && moduleImage.startsWith('data:image')) {
			const file = base64ToFile(moduleImage, 'moduleImage.png')
			payload.append('file', file)
		} else if (moduleImage instanceof File) {
			payload.append('file', moduleImage)
		}

		if (isEdit) {
			payload.append('id', String(module?.id || 0))
			updateMutate.mutate(payload)
		} else {
			creationMutate.mutate(payload)
		}
	}

	const isConfirmButtonDisabled =
		!moduleName ||
		!moduleDescription ||
		creationMutate.isPending ||
		updateMutate.isPending ||
		isDeletePending

	return (
		<DefaultModal
			title={modalTitle}
			subtitle='Personalize o módulo disponibilizado aos colaboradores.'
			content={
				<>
					<ImageInput handleImageChange={handleImageChange} imageUrl={moduleImage} />
					<div className='form-column'>
						<Input
							type='text'
							label='Título'
							value={moduleName}
							onChange={(e) => setModuleName(e.target.value)}
						/>
						<Input
							type='text'
							label='Descrição'
							value={moduleDescription}
							onChange={(e) => setModuleDescription(e.target.value)}
						/>
					</div>
				</>
			}
			footer={
				<>
					{isEdit ? (
						<Button variant='outlined' color='transparent' onClick={handleConfirmation}>
							<div className='modal-button-text'>Deletar</div>
						</Button>
					) : (
						<Button variant='outlined' color='transparent' onClick={() => setVisibility?.(false)}>
							<div className='modal-button-text'>Cancelar</div>
						</Button>
					)}
					<Button
						disabled={isConfirmButtonDisabled}
						variant='filled'
						color='primary'
						onClick={handleSubmit}
					>
						{creationMutate.isPending || updateMutate.isPending ? (
							<div className='modal-button-loader'>
								<Loader />
							</div>
						) : (
							<div className='modal-button-text'>Salvar</div>
						)}
					</Button>
				</>
			}
		/>
	)
}

export default ModuleModal
