import './index.scss'
import { useEffect, useState } from 'react'
import { Card } from '../../../components/Card'
import ModuleModal from './components/ModuleModal'
import { useModal } from '../../../providers/Modal/ModalProvider'
import { Button, ButtonProps } from '../../../components/Button'
import { ReactComponent as EditPencil } from '../../../assets/icons/pecil.svg'
import { useGetAllCourseModules } from '../../../services/core/courses/queries'
import { ModuleDto } from '../../../dtos/ModuleDto'
import { Loader } from '../../../components/Loader'
import { useParams } from 'react-router-dom'
import { DragEndEvent } from '@dnd-kit/core'
import { arrayMove } from '@dnd-kit/sortable'
import DraggableSlider from '../../../components/DraggableSlider'
import { useNavigation } from '../../../providers/Navigation/NavigationProvider'
import { useCourseModuleChangeOrderMutation } from '../../../services/core/courses/mutations'

const OrganizationModules: React.FC = () => {
	const { setModalContent, setVisibility } = useModal()
	const { courseId: courseIdParam } = useParams()
	const [courseId, setCouseId] = useState<number>()
	const [modules, setModules] = useState<ModuleDto[]>([])
	const [isEditingOrder, setIsEditingOrder] = useState<boolean>(false)
	const [originalOrder, setOriginalOrder] = useState<ModuleDto[]>([])

	const { data, isLoading } = useGetAllCourseModules(courseId)

	const { navigateActivityList } = useNavigation()

	const {
		mutate: changeOrderMutate,
		isPending: isChangeOrderPending,
		isSuccess: isChangeOrderSuccess
	} = useCourseModuleChangeOrderMutation(courseId, modules)

	useEffect(() => {
		if (!isEditingOrder) setOriginalOrder([...modules])
	}, [modules, isEditingOrder])

	useEffect(() => {
		if (courseIdParam) setCouseId(parseInt(courseIdParam))
	}, [courseIdParam])

	useEffect(() => {
		if (data) setModules(data)
	}, [data])

	useEffect(() => {
		if (isChangeOrderSuccess) setIsEditingOrder(false)
	}, [isChangeOrderSuccess])

	const handleCreate = () => {
		setModalContent?.(<ModuleModal modalTitle='Criar Módulo' courseId={courseId} />)
		setVisibility?.(true)
	}

	const handleEdit = (module: ModuleDto) => {
		setModalContent?.(<ModuleModal modalTitle='Editando Módulo' module={module} isEdit={true} />)
		setVisibility?.(true)
	}

	const handleOrderCancel = () => {
		setModules([...originalOrder])
		setIsEditingOrder(false)
	}

	const getModulePos = (id: number) => modules.findIndex((m) => m.id === id)

	const clickEnterModule = (module: ModuleDto) => {
		navigateActivityList?.({
			props: {
				courseId,
				moduleId: module.id
			},
			state: { moduleTitle: module.name, moduleDescription: module.description }
		})
	}

	const handleDragEnd = (event: DragEndEvent) => {
		setIsEditingOrder(true)
		const { active, over } = event

		if (!over) return
		if (active.id === over.id) return

		const originalPos = getModulePos(parseInt(active.id as string))
		const newPos = getModulePos(parseInt(over.id as string))

		const newArray = arrayMove(modules, originalPos, newPos)

		setModules(newArray)
	}

	const areButtonsDisabled = isEditingOrder

	const renderCard = (item: ModuleDto) => {
		const buttons: ButtonProps[] = [
			{
				variant: 'filled',
				color: 'primary',
				disabled: areButtonsDisabled,
				onClick: () => clickEnterModule(item),
				children: (
					<div className='card-button-div'>
						<p className='card-button-text'>Entrar</p>
					</div>
				)
			},
			{
				variant: 'filled',
				color: 'gray',
				disabled: areButtonsDisabled,
				onClick: () => handleEdit(item),
				children: (
					<div className='card-button-div'>
						<EditPencil className='card-button-icon' />
						<p className='card-button-text'>Editar</p>
					</div>
				)
			}
		]
		return <Card title={item.name} imageUrl={item.imageUrl} buttons={buttons} />
	}

	return (
		<div className='management-container'>
			<div className='management-top'>
				<h1 className='management-title'>Módulos</h1>
				<p className='management-text'>
					Aqui você poderá criar, editar e reordenar os módulos que os colaboradores usarão para
					seus estudos.
				</p>
			</div>
			<div className='management-middle'>
				<div className='management-cards'>
					{isLoading ? (
						<Loader />
					) : (
						<DraggableSlider
							items={modules}
							handleDragEnd={handleDragEnd}
							renderItem={renderCard}
						/>
					)}
				</div>
				<div className='management-bottom'>
					<div className='management-button'>
						{isEditingOrder ? (
							<>
								<Button
									variant='outlined'
									color='transparent'
									size='big'
									onClick={handleOrderCancel}
									disabled={isChangeOrderPending}
								>
									<p className='management-button-text-add'>Cancelar</p>
								</Button>
								<Button
									disabled={isChangeOrderPending}
									variant='filled'
									color='primary'
									size='big'
									onClick={changeOrderMutate}
								>
									{isChangeOrderPending ? (
										<Loader />
									) : (
										<p className='management-button-text-add'>Salvar ordem</p>
									)}
								</Button>
							</>
						) : (
							<Button
								disabled={areButtonsDisabled}
								variant='filled'
								color='primary'
								size='big'
								onClick={handleCreate}
							>
								<p className='management-button-text-add'>Criar</p>
							</Button>
						)}
					</div>
				</div>
			</div>
		</div>
	)
}

export default OrganizationModules
