import React, { createContext, ReactNode, useContext, useEffect, useState } from 'react'
import { UserProfileDto } from '../../dtos/UserDto'
import authStorage from '../../utils/stores/auth'
import { jwtDecode } from 'jwt-decode'
import { Loader } from '../../components/Loader'

interface AuthProviderProps {
	user: UserProfileDto | null
	isLogged: (() => boolean) | null
	setAuthToken: ((token?: string) => void) | null
	logout: (() => void) | null
}

const AuthContext = createContext<AuthProviderProps>({
	user: null,
	isLogged: null,
	setAuthToken: null,
	logout: null
})

const AuthProvider: React.FC<{ children?: ReactNode }> = ({ children }) => {
	const [user, setUser] = useState<UserProfileDto | null>(null)
	const [token, setToken] = useState<string | undefined>()

	useEffect(() => {
		const storageToken = authStorage.getAuthToken()
		if (storageToken) setToken(storageToken)
	}, [])

	useEffect(() => {
		if (token) {
			const decoded = jwtDecode(token) as UserProfileDto
			setUser({ ...decoded })
		} else {
			setUser(null)
		}
	}, [token])

	const isLogged = () => !!authStorage.getAuthToken()

	const setAuthToken = (token?: string) => {
		setToken(token)
		if (token) {
			authStorage.setAuthToken(token)
		} else {
			authStorage.removeAuthToken()
		}
	}

	const logout = () => {
		setToken('')
		authStorage.removeAuthToken()
		window.location.reload()
	}

	const value = {
		user,
		isLogged,
		setAuthToken,
		logout
	}

	return (
		<AuthContext.Provider value={value}>
			{user || !isLogged() ? children : <Loader />}
		</AuthContext.Provider>
	)
}

export { AuthProvider }

export function useAuth() {
	return useContext(AuthContext)
}
