import React, { createContext, ReactNode, useContext } from 'react'
import { useAuth } from '../Auth/AuthProvider'
import { useNavigate } from 'react-router-dom'
import PAGES from '../../utils/contants/pages'
import { UserRole } from '../../dtos/UserDto'
import { QuestionTypes } from '../../pages/Common/Activities/components/card-activity'

interface NavigatorFunction<T> {
	(params?: T): void
}

interface NavigateActivityListProps {
	courseId?: number
	moduleId?: number
}

interface NavigationProviderProps {
	navigateHome: NavigatorFunction<undefined> | null
	navigateLogin: NavigatorFunction<undefined> | null
	navigateAdminDashboard: NavigatorFunction<undefined> | null
	navigateAreas: NavigatorFunction<undefined> | null
	navigateCoursesCollaborator: NavigatorFunction<undefined> | null
	navigateCreateEditActivity: NavigatorFunction<{
		activityId?: number
		state?: {
			questionType: QuestionTypes
			moduleId: number
			courseId: number
			currentIndex?: number
		}
	}> | null
	navigateActivityList: NavigatorFunction<{
		props?: NavigateActivityListProps
		state?: { isLastModule?: boolean; moduleTitle?: string; moduleDescription?: string }
	}> | null
	navigateCourses: NavigatorFunction<{ areaId?: number }> | null
	navigateModules: NavigatorFunction<{ courseId: number }> | null
	navigateActivities: NavigatorFunction<{ activityId: number }> | null
}

const NavigationContext = createContext<NavigationProviderProps>({
	navigateHome: null,
	navigateLogin: null,
	navigateAdminDashboard: null,
	navigateAreas: null,
	navigateCoursesCollaborator: null,
	navigateCourses: null,
	navigateModules: null,
	navigateCreateEditActivity: null,
	navigateActivityList: null,
	navigateActivities: null
})

const NavigationProvider: React.FC<{ children?: ReactNode }> = ({ children }) => {
	const navigate = useNavigate()
	const { user, isLogged } = useAuth()

	const navigateHome = () => {
		if (!isLogged?.()) return navigate(PAGES.home)
		if (user?.role === UserRole.ADMIN) return navigate(PAGES.organizationHome)
		if (user?.role === UserRole.COLLABORATOR) return navigate(PAGES.collaboratorHome)
	}

	const navigateLogin = () => {
		if (isLogged?.()) return navigateHome()
		return navigate(PAGES.login)
	}

	const navigateCreateEditActivity = (params?: {
		activityId?: number
		state?: { questionType: QuestionTypes; moduleId?: number; courseId: number }
	}) => {
		return navigate(
			`${PAGES.organizationEditCreateActivities}${params?.activityId !== undefined ? '?activityId=' + params?.activityId : ''}`,
			{ state: params?.state }
		)
	}

	const navigateActivityList = (params?: {
		props?: NavigateActivityListProps
		state?: { isLastModule?: boolean; moduleTitle?: string; moduleDescription?: string }
	}) => {
		if (user?.role === UserRole.ADMIN)
			return navigate(
				`${PAGES.organizationCourses}/${params?.props?.courseId}/modulos/${params?.props?.moduleId}/atividades`,
				{ state: params?.state }
			)
		if (user?.role === UserRole.COLLABORATOR)
			return navigate(
				`${PAGES.colaboratorCourses}/${params?.props?.courseId}/modulos/${params?.props?.moduleId}/atividades`,
				{ state: params?.state }
			)
	}

	const navigateAdminDashboard = () => {
		return navigate(PAGES.admin)
	}

	const navigateAreas = () => {
		return navigate(PAGES.areas)
	}

	const navigateCoursesCollaborator = () => {
		return navigate(PAGES.collaboratorCourses)
	}

	const navigateCourses = (params?: { areaId?: number }) => {
		if (user?.role === UserRole.ADMIN)
			return navigate(`${PAGES.organizationCourses}?areaId=${params?.areaId}`)
		if (user?.role === UserRole.COLLABORATOR)
			return navigate(`${PAGES.collaboratorCourses}?areaId=${user.areaId}`)
	}

	const navigateModules = (params?: { courseId?: number }) => {
		if (user?.role === UserRole.ADMIN)
			return navigate(`${PAGES.organizationCourses}/${params?.courseId}/modulos`)
		if (user?.role === UserRole.COLLABORATOR)
			return navigate(`${PAGES.collaboratorCourses}/${params?.courseId}/modulos`)
	}

	const value = {
		navigateHome,
		navigateLogin,
		navigateAdminDashboard,
		navigateAreas,
		navigateCoursesCollaborator,
		navigateCourses,
		navigateModules,
		navigateCreateEditActivity,
		navigateActivityList,
		navigateActivities: null
	}

	return <NavigationContext.Provider value={value}>{children}</NavigationContext.Provider>
}

export { NavigationProvider }

export function useNavigation() {
	return useContext(NavigationContext)
}
