import { useMutation } from '@tanstack/react-query'
import { courseApi } from '.'
import { ModuleChangeOrderDto, ModuleDto } from '../../../dtos/ModuleDto'
import { queryClient } from '../../query-client'
import QUERY_KEYS from '../../../utils/contants/queries'
import { CourseDto } from '../../../dtos/CourseDto'
import {
	ActivitiesToAdd,
	ActivityDto,
	ActivityResponseDto,
	ActivityUpdateListDtoRequestPayload,
	CourseFeedbackDto
} from '../../../dtos/ActivityDto'

export const useCourseCreateMutation = () =>
	useMutation({
		mutationFn: (payload: FormData) =>
			courseApi
				.post('', payload, {
					headers: {
						'Content-Type': 'multipart/form-data'
					}
				})
				.then((res) => res.data as CourseDto),
		onSuccess: () => queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course] })
	})

export const useCourseUpdateMutation = () =>
	useMutation({
		mutationFn: (payload: FormData) =>
			courseApi
				.patch(`${payload.get('id')}`, payload, {
					headers: {
						'Content-Type': 'multipart/form-data'
					}
				})
				.then((res) => res.data as CourseDto),
		onSuccess: () => queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course] })
	})

export const useCourseModuleCreateMutation = () =>
	useMutation({
		mutationFn: (payload: FormData) =>
			courseApi
				.post(`${payload.get('courseId')}/modules`, payload, {
					headers: {
						'Content-Type': 'multipart/form-data'
					}
				})
				.then((res) => res.data as ModuleDto),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course, QUERY_KEYS.module] })
	})

export const useCourseModuleUpdateMutation = () =>
	useMutation({
		mutationFn: (payload: FormData) =>
			courseApi
				.patch(`${payload.get('courseId')}/modules/${payload.get('id')}`, payload, {
					headers: {
						'Content-Type': 'multipart/form-data'
					}
				})
				.then((res) => res.data as ModuleDto),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course, QUERY_KEYS.module] })
	})

export const useCourseModuleDeleteMutation = (courseId?: number, moduleId?: number) =>
	useMutation({
		mutationFn: () =>
			courseApi.delete(`${courseId}/modules/${moduleId}`).then((res) => res.data as ModuleDto),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course, QUERY_KEYS.module] })
	})

export const useCourseModuleChangeOrderMutation = (
	courseId?: number,
	modules?: ModuleChangeOrderDto[]
) =>
	useMutation({
		mutationFn: () =>
			courseApi
				.patch(`${courseId}/modules/order`, { modules })
				.then((res) => res.data as ModuleDto[]),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course, QUERY_KEYS.module] })
	})

export const useCourseDeleteMutation = (courseId?: number) =>
	useMutation({
		mutationFn: () => courseApi.delete(`${courseId}`).then((res) => res.data as CourseDto),
		onSuccess: () => queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.course] })
	})

export const useCourseModuleAddActivity = (
	moduleId: number,
	payload: ActivitiesToAdd,
	courseId: number
) =>
	useMutation({
		mutationFn: () =>
			courseApi
				.post(`${courseId}/modules/${moduleId}/activities`, payload)
				.then((res) => res.data as ActivityDto[]),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.activity, QUERY_KEYS.module] })
	})

export const useCourseModuleDeleteActivity = (moduleId: number, activityId: number) =>
	useMutation({
		mutationFn: () =>
			courseApi
				.delete(`modules/${moduleId}/activities/${activityId}`)
				.then((res) => res.data as ActivityResponseDto[]),
		onSuccess: () =>
			queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.activity, QUERY_KEYS.module] })
	})

export const useCourseModuleUpdateActivity = (
	moduleId: number,
	activityToAddPayload: ActivityUpdateListDtoRequestPayload,
	courseId: number
) =>
	useMutation({
		mutationFn: () =>
			courseApi
				.patch(`${courseId}/modules/${moduleId}/activities`, activityToAddPayload)
				.then((res) => res.data as ActivityDto[])
	})

export const useCourseModuleActivityFeedback = (courseId: number, rating: CourseFeedbackDto) =>
	useMutation({
		mutationFn: () =>
			courseApi.post(`${courseId}/feedback`, rating).then((res) => res.data as CourseFeedbackDto),
		onSuccess: () => queryClient.invalidateQueries({ queryKey: [QUERY_KEYS.activity] })
	})
